; Wolfram Hildebrandt
; wolfi-home@lycos.de



.nolist
.include "m8def.inc"
.list



.equ	E	= 0
.equ	RW	= 1
.equ	RS	= 2

.equ	port_display = PORTD
.equ	ddr_display	= DDRD
.equ	pin_display	= PIND


.equ	command=0x65



.org 0x0000
rjmp	reset

reset:
; Stackpointer
ldi	r16, low(RAMEND)
ldi	r17, high(RAMEND)
out	SPL, r16
out SPH, r17

ldi	r16, 0b00000000
out	DDRB, r16
ldi	r16, 0b00000100
out	DDRC, r16

ldi	r16, 0b11111111
out	ddr_display, r16		; Display Port auf Ausgang

ldi	r16, 0x00
out port_display, r16




rcall	LCD_init

ldi		r16, 'A'
sts		command, r16
rcall	send_data
ldi		r16, 'D'
sts		command, r16
rcall	send_data

ende:
rjmp ende

;++++++++++++++++++++++++++++++++++++++++++++++++++++++
;--------------------------------------------------------------------
LCD_init:
rcall delay_40ms

ldi	r16, 0b00110000
sts command, r16
rcall send_1_nibble
rcall delay_2ms

ldi	r16, 0b00110000
sts command, r16
rcall send_1_nibble
rcall delay_30us

ldi	r16, 0b00110000
sts command, r16
rcall send_1_nibble
rcall delay_30us

rcall busy_check
ldi	r16, 0b00100000
sts command, r16
rcall send_1_nibble
rcall delay_30us		; function set

ldi	r16, 0b00101001		
sts command, r16		
rcall send
rcall delay_30us		; function set 4 bit, 2line display

ldi	r16, 0b00011100
sts command, r16
rcall send
rcall delay_30us		; bias

ldi	r16, 0b01010010
sts command, r16
rcall send
rcall delay_30us		; 3v booster aus, power control

ldi	r16, 0b01101001
sts command, r16
rcall send
rcall delay_30us		; follover control

ldi	r16, 0b01110100
sts command, r16
rcall send
rcall delay_30us		; contrast set

ldi	r16, 0b00001111
sts command, r16
rcall send
rcall delay_30us		; display on

ldi	r16, 0b00000001
sts command, r16
rcall send
rcall delay_2ms	     	; clear display

ldi	r16, 0b00000110
sts command, r16
rcall send
rcall delay_30us		; entry mode auto increment



ret
;--------------------------------------------------------------------


send:

rcall	busy_check

lds r16, command
andi r16, 0xF0
out	port_display, r16
cbi	port_display, rs
cbi	port_display, rw
sbi	port_display, e
nop
nop
nop
nop
nop
cbi	port_display, e

lds r16, command
swap r16
sts	command, r16

send_1_nibble:
lds r16, command
andi r16, 0xF0
out	port_display, r16
cbi	port_display, rs
cbi	port_display, rw
sbi	port_display, e
nop
nop
nop
nop
nop
cbi	port_display, e

ret



send_data:

rcall	busy_check

lds r16, command
andi r16, 0xF0
out	port_display, r16
sbi	port_display, rs
cbi	port_display, rw
sbi	port_display, e
nop
nop
nop
nop
nop
cbi	port_display, e

lds r16, command
swap r16

andi r16, 0xF0
out	port_display, r16
sbi	port_display, rs
cbi	port_display, rw
sbi	port_display, e
nop
nop
nop
nop
nop
cbi	port_display, e

ret



;--------------------------------------------------------------------
busy_check:
ldi	r18,0b00001111
out	ddr_display, r18
nop
nop

marke:
cbi port_display, rs
sbi port_display, rw
sbi port_display, e
nop
nop
in  r17, pin_display
cbi	port_display, e

nop
nop
sbi port_display, e
nop
nop
nop
nop
nop
cbi	port_display, e

SBRC r17, 7 ; berspringe wenn Bit 0 Null ist
RJMP marke

ldi	r18,0b11111111
out	ddr_display, r18

ret
;--------------------------------------------------------------------

;--------------------------------------------------------------------
delay_40ms:
; ============================= 
;    delay loop generator 
;     40000 cycles:
; ----------------------------- 
; delaying 39999 cycles:
          ldi  R17, $43
WGLOOP0:  ldi  R18, $C6
WGLOOP1:  dec  R18
          brne WGLOOP1
          dec  R17
          brne WGLOOP0
; ----------------------------- 
; delaying 1 cycle:
          nop
; ============================= 
ret
;--------------------------------------------------------------------
delay_30us:
; ============================= 
;    delay loop generator 
;     30 cycles:
; ----------------------------- 
; delaying 30 cycles:
          ldi  R17, $0A
WGLOOP0b:  dec  R17
          brne WGLOOP0b
; ============================= 

ret
;--------------------------------------------------------------------
delay_2ms:
; ============================= 
;    delay loop generator 
;     2000 cycles:
; ----------------------------- 
; delaying 1998 cycles:
          ldi  R17, $03
WGLOOP0c:  ldi  R18, $DD
WGLOOP1c:  dec  R18
          brne WGLOOP1c
          dec  R17
          brne WGLOOP0c
; ----------------------------- 
; delaying 2 cycles:
          nop
          nop
; ============================= 

ret



